#pragma once

#include <QtCore/QObject>
#include <QtWebSockets/QWebSocket>
#include <QString>
#include <QQueue>
#include <QtCore/QThread>
#include <iostream>
#include "SocketHandler.h"
#include <wschatcom/Message.h>

/**
 * Represents an interface between a GUI and a server capable of sending and receiving messages via websocket
 * on a new thread. The client itself runs on the same thread as the calling GUI,
 * but its websocket is moved to a different thread and the two communicate via slots and signals. The client also
 * provides signals for the GUI to register received messages or to be notified once a message has been sent
 */
class Client : public QObject
{
    Q_OBJECT
public:
    explicit Client(const QUrl &url, QObject *parent = nullptr);
    void add_message(const Message& message);
    void change_server_url(const QUrl &url) { emit change_server_url_signal(url); }

    Client(const Client& other);
    Client& operator=(const Client& other);

    ~Client() override;

signals:
    void message_added(QString str_msg);
    void message_received(Message message);
    void change_server_url_signal(const QUrl &url);
    void connected();
    void disconnected();

    /**
     * Once this signal is emitted <m_handler> closes the socket and turns of its timer,
     * and thus, is permanently disconnected from the server
     */
    void destroy_handler();
    void recreate_handler();

public slots:
    void add_message_slot(QString message);

private slots:
    void on_message_received(QString message);


private:
    SocketHandler m_handler;
    QThread m_handler_thread;
    bool m_logged_out = false;
};
