#pragma once

#include <QtCore/QObject>
#include <QtWebSockets/QWebSocket>
#include <QString>
#include <wschatcom/Message.h>

/**
 * SocketHandler is a simple wrapper for QWebSocket, which provides no extra functionality,
 * but helps with multithreading of the socket
 *
 * Main reason for this wrapper is this error:
 * QSocketNotifier: Socket notifiers cannot be enabled or disabled from another thread
 * Meaning client itself cannot simply connect its signals to QWebSocket's and then move the socket to a different thread
 * Only a class in the same thread can communicate with the QWebSocket class via signals.
 * Fortunately, this is possible with custom classes, therefore this wrapper exists
 */
class SocketHandler : public QObject
{
    Q_OBJECT
public:
    explicit SocketHandler(const QUrl &url, QObject *parent = nullptr);
    QUrl url() const { return m_url; }
    const QQueue<QString>& unsent_messages() const { return m_to_send; }
    friend class Client;

signals:
    void socket_disconnected();
    void message_received(QString message);
    void message_sent();
    void socket_connected();

public slots:
    void on_message_added(QString str);
    void send_messages();
    void try_reconnect();
    void on_change_server_url(const QUrl &url);
    void on_destroy();
    void on_recreate();
    void on_message_received(QString message);

private slots:
    void on_socket_connected();
    void on_socket_disconnected();

private:
    bool m_connected = false;
    QUrl m_url;
    QWebSocket m_web_socket;
    QTimer m_reconnect_timer;
    QQueue<QString> m_to_send;
    QString m_login_msg = "";
};
