#include <iostream>
#include <QTextStream>
#include <QtCore>
#include "../Client.h"

/*
 * *******************************
 * ********* UNIX ONLY ***********
 * *******************************
 */

const int port = 5055;
const QString create_listener_cmd = "python3";
const QStringList args = QStringList() <<  "../WebsocketServer.py " + QString::number(port);
//const std::string create_listener_cmd = "python3 ../WebsocketServer.py " + std::to_string(port) + " &";
//const std::string kill_listener_cmd = "kill `pgrep -f \"python3 ../WebsocketServer.py\"`";

bool check_messages(QList<Message> messages, QString filename="messages.txt")
{
    QFile file(filename);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return false;

    QByteArray message;
    int c = 0;
    while (!file.atEnd())
    {
        QByteArray line = file.readLine();
        message += line;
        if (line[0] == '}')
        {
            if (!messages[c].equals_without_time(Message::from_json_byte(message)))
            //if (!bool(QString(message) == messages[c].get_raw_string()))
            //if (Message::from_json_string(message) == messages[c])
                return false;
            ++c;
            message.clear();
        }
    }

    return message.isEmpty() && c == messages.size();
}

/**
 * This is the single non automated test
 */
void manual_testing()
{
    QProcess p;
  p.start(create_listener_cmd, args);

    Client client(QUrl(QString("ws://localhost:") + QString::number(port)));

    QTextStream cout(stdout);
    QTextStream cin(stdin);

    QString msg;
    QThread::sleep(1);
    cout << "Type your message: ";
    cout.flush();
    Client c(QUrl(QStringLiteral("ws://localhost:5055")));
    c = client;
    cout << "Type your ffff: ";
    cout.flush();


    Message m = Message::create_logout_message("glhf");
    auto x = m;
    m = Message::create_logout_message("ty");


    QThread::sleep(2);

    while (cin.readLineInto(&msg)) {
        client.add_message(Message::create_text_message("Alice", "Bob", msg));
        QThread::sleep(2);
        //cin.readLineInto(&msg);
        cout << "Choose option: send message [1], change server url[2], end[3]\n";
        cout.flush();
        cin.readLineInto(&msg);
        while (msg != QStringLiteral("3") && msg != QStringLiteral("2") && msg != QStringLiteral("1"))
        {
            cout << "Wrong input!\n";
            cout.flush();
            cout << "Choose option: send message [1], change server url[2], end[3]\n";
            cout.flush();
            cin.readLineInto(&msg);
        }
        if (msg == QStringLiteral("2")) {
            cout << "Type url: ";
            cout.flush();
            cin.readLineInto(&msg);
            cout << "\n";
            cout.flush();
            client.change_server_url(msg);
        }

        if (msg == QStringLiteral("3"))
        {
            break;
        }

            cout << "Type your message: ";
        cout.flush();
    }
    p.kill();
    p.waitForFinished();
    //system("kill `pgrep -f \"python3 ../WebsocketServer.py\"`");
}


void test_simple_send()
{
    QProcess p;
    p.start(create_listener_cmd, args);
    Client client(QUrl(QString("ws://localhost:") + QString::number(port)));

    auto simple_text = Message::create_text_message("Alice", "Bob", "Hello, this is from Alice!");
    auto simple_text2 = Message::create_text_message("Bob", "Alice", "Hi from Bob!");

    client.add_message(simple_text);
    client.add_message(simple_text2);
    QThread::sleep(5); // wait for writing


    std::cout << "Simple message sending " << (check_messages({simple_text, simple_text2}) ? "OK" : "NOK")
    << std::endl;

    p.kill();
    p.waitForFinished();

}

void test_send_after_reconnection()
{
    QProcess p;
    p.start("python3", QStringList() <<  "../WebsocketServer.py");
    Client client(QUrl(QString("ws://localhost:") + QString::number(port + 1)));

    auto simple_text = Message::create_text_message("Alice", "Bob", "Hello, this is from Alice!");
    auto simple_text2 = Message::create_text_message("Bob", "Alice", "Hi from Bob!");

    client.add_message(simple_text);
    client.add_message(simple_text2);
    QThread::sleep(5); // wait for writing

    if (!check_messages({}))
    {
        std::cout << "Message sending after reconnection failed, "
                     "bcs messages were correctly sent, though the client was disconnected"
                  << std::endl;
    }
    else
    {
        client.change_server_url(QUrl(QString("ws://localhost:") + QString::number(port)));
        QThread::sleep(5); // wait for writing

        std::cout << "Message sending after reconnection "
                  << (check_messages({simple_text, simple_text2}) ? "OK" : "NOK")
                  << std::endl;
    }
    p.kill();
    p.waitForFinished();
}


/**
 * The automated tests (test_simple_send and test_send_after_reconnection) are a bit faulty and dont work for the first launch of the main function
 * Somehow, the python scripts fails/takes too long to create the messages.txt file and the first test fails
 *
 * Results of the maunal_testing() can be found in build direcotry in messages.txt
 */
int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);
    test_simple_send();
    test_send_after_reconnection();
    manual_testing();
}
