#pragma once

#include <unordered_map>

/**
 * This is simple implementation of the bidirectional map structure.
 * The main point is that this structure allows us to access A when B is used
 * as a key and access B when A is used as a key.
 * This implies that A and B are unique.
 */
template<typename A, typename B>
class BiMap
{
public:
    BiMap() = default;

    void insert(A first, B second)
    {
        m_first.insert({first, second});
        m_second.insert({second, first});
    }

    void emplace(A first, B second) {
        m_first.emplace(first, second);
        m_second.emplace(second, first);
    }

    A getFirst(const B &v)
    {
        return m_second[v];
    }

    B getSecond(const A &v)
    {
        return m_first[v];
    }

    void remove(const A &v)
    {
        auto s = m_first[v];
        m_first.erase(v);
        m_second.erase(s);
    }

    void remove(const B &v)
    {
        auto s = m_second[v];
        m_first.erase(s);
        m_second.erase(v);
    }

    bool contains(const A &v)
    {
        return m_first.count(v);
    }

    bool contains(const B &v)
    {
        return m_second.count(v);
    }
private:
    std::unordered_map<A, B> m_first;
    std::unordered_map<B, A> m_second;
};