#pragma once

#include "bimap.hpp"

#include <mutex>

#include <QString>
#include <QSqlDatabase>
#include <QSqlQuery>
#include <QFile>
#include <QThreadStorage>
#include <QSqlError>
#include <QDebug>

class QWebSocket;

/**
 * Handles database.
 */
class DatabaseWrapper
{
public:
    static DatabaseWrapper &instance()
    {
        static DatabaseWrapper i;
        return i;
    }

    static void setDatabasePath(const QString &path)
    {
        filename = path;
    }

    bool containsUser(QString user);

    bool registerUser(QString username, QString password);

    bool authenticateUser(QString username, QString password, QWebSocket *user);

    void logoutUser(QWebSocket *user);

    QString getUser(QWebSocket *user);

    QWebSocket *getSocket(const QString &user);

    bool addFriendship(const QString &lhs, const QString &rhs);

    bool removeFriendship(const QString &lhs, const QString &rhs);

    std::vector<QString> getFriends(const QString &user);

    void saveUndeliveredMessage(const QString &user, const QString &msg);

    std::vector<QString> getUndeliveredMessages(const QString &user);

    DatabaseWrapper(const DatabaseWrapper &) = delete;

    DatabaseWrapper(DatabaseWrapper &&) = delete;

    DatabaseWrapper &operator=(const DatabaseWrapper &) = delete;

    DatabaseWrapper &operator=(DatabaseWrapper &&) = delete;

private:
    static QString filename;

    DatabaseWrapper();

    QSqlDatabase getThreadDatabase();

    bool executeQuery(const QString &query);

    QThreadStorage<QString> m_threadStorage;
    QSqlDatabase m_db;
    BiMap<QWebSocket*, std::string> m_auth_users;
    std::mutex m_bimapMutex;
};