#pragma once

#include <QtCore>
#include <wschatcom/Message.h>
#include <Client.h>

/**
 * DISCLAIMER
 * Sometimes the tests might not kill the server process and consequently all further tests will not run properly
 */
struct Tester : public QObject
{
    Q_OBJECT
public:
    explicit Tester(QObject* parent = nullptr);

    // this class is supoosed to be only instantiated once
    Tester(const Tester& other) = delete;
    Tester& operator=(const Tester& other) = delete;
    void finish();

    Q_INVOKABLE void test_all();
    Q_INVOKABLE void test_register();

    /**
     * Clients must be preregistered
     */
    void test_login();

    void test_friend_request();

    /**
     * Sending message to user not in your friendslist is impossible due to GUI's functionality
     * and so it does not have to be handled herer
     */
    void test_message_sending();
    void register_clients();
    void login_clients();
    //void test_unfriend();
    static void check_message(const QString& client_name, const Message& actual, const Message& correct);

signals:
    void client1_send(QString m);
    void client2_send(QString m);


public slots:
    void on_client1_received(Message m);
    void on_client2_received(Message m);


private:
    const QString cl1_name = "Claire1";
    const QString cl1_pass = "1234";
    const QString cl2_name = "John1";
    const QString cl2_pass = "458";

    const int m_server_port = 5052;
    const QUrl m_server_url = QUrl("ws://localhost:" + QString::number(m_server_port));
    QProcess m_server_process;
    QThread m_thread1;
    QThread m_thread2;
    Client m_client1;
    Client m_client2;
    QQueue<QString> m_info;
    QQueue<Message> m_correct_response_cl1;
    QQueue<Message> m_correct_response_cl2;
    QQueue<Message> m_received_messages_cl1;
    QQueue<Message> m_received_messages_cl2;
};

